#include <gadgets/texteditor.h>

#define __NOLIBBASE__
#include <proto/exec.h>

#define __CLIB_PRAGMA_LIBCALL
#include <proto/alib.h>
#include <proto/utility.h>

#include "/filetypes.h"
#include "m68k.h"

extern struct Library *SysBase;
extern struct Library *UtilityBase;
extern struct Library *TextFieldBase;

static void init(APTR userData);
static void terminate(struct Filetype *fType);
static Object *settingsGadget(struct Filetype *fType, ULONG i);
static STRPTR settingsTitle(struct Filetype *fType, ULONG i);
static void applySettings(struct Filetype *fType);
void setSettingsFromGUI(struct Filetype *fType);
void setGUIFromSettings(struct Filetype *fType, struct Window *window);
void disposeGadgets(struct Filetype *fType);
void processGadgetUp(struct Filetype *fType, UWORD gadgetId, struct Window *window);
BOOL saveHighlightSettings(struct IFFHandle *iff, struct ContextNode *cn, APTR data);
BOOL loadHighlightSettings(struct IFFHandle *iff, struct ContextNode *cn, APTR data);

#define INT_VERSION    1
#define INT_REVISION   0

struct Plugin exampleFiletypePlugin =
{
  NULL, // we don't want to chain more filetypes in the same plugin file
  PLUGIN_VERSION,
  MASTER_VERSION,
  INT_VERSION,
  INT_REVISION,
  0,
  0,
  (APTR)init
};

static void terminate(struct Filetype *fType)
{
    if (TextFieldBase)
        CloseLibrary(TextFieldBase);

    if (UtilityBase)
        CloseLibrary(UtilityBase);
}

static Object *settingsGadget(struct Filetype *fType, ULONG i)
{
    // you can return a reaction gadget that will be shown as a page in the dialog
    // however make sure to return a corresponding string in the settingsTitle function
    // and be sure to return NULL for any i higher than your gadgets

    // this call is also the que to copy the settings to a workingcopy of your settings
    return NULL;
}

static STRPTR settingsTitle(struct Filetype *fType, ULONG i)
{
    // see above and be sure to return NULL for any i higher than your gadgets
    return NULL;
}

static void applySettings(struct Filetype *fType)
{
    // Now is the time to take your internal setting values and move them to the place
    // where your highlighter get's it's values
    // This is the function that get's called when the user presses "ok" in the settings
    // dialog
}

void setSettingsFromGUI(struct Filetype *fType)
{
    // This is the function that get's called when the user moves away from your filetype
    // in the dialog - it doesn't mean they should be used
    // set working copy settings from gadgets
    // Do not set the real settings - that is only something you should do in applySettings
}

void setGUIFromSettings(struct Filetype *fType, struct Window *window)
{
    // if you have your own gadgets, now is the chance to set their
    // values from the working copy settings
    // This is the function that is called when your filetype gets selected
}

void disposeGadgets(struct Filetype *fType)
{
    // please destroy the gadgets you have created
    // this function will be called when the prefs settings dialog closes
    // You will come here both on "ok" and "Cancel"
}

void processGadgetUp(struct Filetype *fType, UWORD gadgetId, struct Window *window)
{
    // so that you can react to the user pressing your gadgets
}

BOOL saveHighlightSettings(struct IFFHandle *iff, struct ContextNode *cn, APTR data)
{
    struct Filetype *fType = (struct Filetype *)data;
    //save you highlighters current settings to file
    return TRUE; //success
}

BOOL loadHighlightSettings(struct IFFHandle *iff, struct ContextNode *cn, APTR data)
{
    struct Filetype *fType = (struct Filetype *)data;
    //load you highlighters current settings from file
    return TRUE; //success
}

/* ORIGINAL
ULONG __saveds __asm highlightBlock(register __a0 struct Hook *h,register  __a2 VOID *o, register __a1 VOID *msg)
{
    struct HighlightMessage *hm = (struct HighlightMessage *)msg;
    STRPTR text = hm->Text;
    UWORD state = hm->StatusOfPrevBlock;
    UWORD start = 0;
    UWORD end = strlen(text);

    // if line is longer than 10 chars we paint the entire line with color 3
    if (end > 10)
        HighlightSetFormat(o, start, end, (34<<8)|TBSTYLE_SETCOLOR|TBSTYLE_ITALIC);

    return (ULONG)state + 1L;
}
*/

/* V001
ULONG __saveds __asm highlightBlock(register __a0 struct Hook *h,register  __a2 VOID *o, register __a1 VOID *msg)
{
	struct MessageInfo messageInfo;
	InitMessageInfo(&messageInfo, (struct HighlightMessage *)msg);

	// if line is longer than 10 chars we paint the entire line with color 3
	if (messageInfo.end > 10)
		HighlightSetFormat(o, messageInfo.start, messageInfo.end, (34<<8)|TBSTYLE_SETCOLOR|TBSTYLE_ITALIC);

	return messageInfo.state + 1L;
}
*/

ULONG __saveds __asm highlightBlock(register __a0 struct Hook *hook,register  __a2 VOID *obj, register __a1 VOID *msg)
{
	return FormatTest2(hook, obj, msg);
}

void openLibs(void)
{
    SysBase  = (*((struct Library **) 4));

    if (! (UtilityBase = OpenLibrary("utility.library",47)))
        return;

    if (! (TextFieldBase = OpenLibrary("gadgets/texteditor.gadget",47)))
        return;
}

static void init(APTR userData)
{
	struct Filetype *fType = (struct Filetype *)userData;
    struct Hook *hook = (struct Hook *)AllocVec(sizeof(struct Hook), MEMF_CLEAR);

    openLibs();

    fType->pluginData = (ULONG)1234567; // any number or pointer of our choosing
	fType->highlighterHook = hook;
    fType->typeName = "example";
    fType->autoFileTypes = "example";
	fType->terminate = terminate;
	fType->settingsTitle = settingsTitle;
	fType->settingsGadget = settingsGadget;
	fType->processGadgetUp = processGadgetUp;
	fType->setSettingsFromGUI = setSettingsFromGUI;
	fType->setGUIFromSettings = setGUIFromSettings;
	fType->applySettings = applySettings;
	fType->saveSettings = saveHighlightSettings;
	fType->loadSettings = loadHighlightSettings;
	fType->disposeGadgets = disposeGadgets;

    fType->name = "Example";

	hook->h_Entry    = (ULONG (*)()) highlightBlock;
	hook->h_SubEntry = NULL;
    hook->h_Data = 1234567;
    
    StaticInitMessagePartsInfo();
}
